from django.http import HttpRequest
from django.http import HttpResponse
from django.http import HttpResponseServerError
from django.shortcuts import render
from sentry_sdk import last_event_id


# noinspection PyUnusedLocal
def collect_500_error_user_feedback_view(request: HttpRequest, *args, **argv) -> HttpResponseServerError:
    return render(
        request,
        template_name='error_handler/500.html',
        context={'sentry_event_id': last_event_id()},
        status=500,
    )


# noinspection PyUnusedLocal
def not_found_404_view(request: HttpRequest, exception: Exception) -> HttpResponse:
    return render(
        request,
        template_name='error_handler/404.html',
        status=404,
    )



from aldryn_sso.models import AldrynCloudUser
from django.contrib import admin
from djangocms_helpers.fixes.aldryn_sso_admin import CustomAldrynSsoAdmin


admin.site.unregister(AldrynCloudUser)  # https://github.com/divio/aldryn-sso/issues/45
admin.site.register(AldrynCloudUser, CustomAldrynSsoAdmin)



from typing import Callable

from cms.cms_toolbars import PageToolbar
from cms.utils import page_permissions


def monkeypatch_cms_has_publish_permission() -> Callable:

    def has_publish_permission(self: PageToolbar) -> bool:
        is_has_publish_permission = False

        is_page_model_available = self.page is not None
        if is_page_model_available:
            is_has_publish_permission = page_permissions.user_can_publish_page(
                self.request.user,
                page=self.page,
                site=self.current_site,
            )

        is_need_to_check_dirty_static_placeholders_perms = (
            (is_has_publish_permission or not is_page_model_available) and self.statics
        )
        if is_need_to_check_dirty_static_placeholders_perms:
            is_has_publish_permission = all(sp.has_publish_permission(self.request) for sp in self.dirty_statics)

        return is_has_publish_permission

    return has_publish_permission


PageToolbar.has_publish_permission = monkeypatch_cms_has_publish_permission()



from aldryn_sso.admin import AldrynCloudUserAdmin
from django.urls import reverse
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _


class CustomAldrynSsoAdmin(AldrynCloudUserAdmin):
    """
    Why?
    - Read https://github.com/divio/aldryn-sso/issues/45
    - sometimes, when login in via Divio, aldryn-sso complains about `duplicate key value violates unique constraint "backend_auth_user_email_key"` 
    - in that case you need the aldryn-sso admin panel to connect the exising user to your custom user model
    - you're welcome!
    """

    def linked_user(self, obj):
        html_link = '<a href="{}">{}</a>'.format(
            reverse('admin:backend_auth_user_change', args=[obj.pk]),
            obj.user,
        )
        return mark_safe(html_link)


    linked_user.short_description = _('User')
